@@ -1,18 +1,36 @@
-# $Id: RankCorrelation.pm,v 1.22 2004/08/13 23:41:45 gene Exp $
+# $Id: RankCorrelation.pm,v 1.26 2007/03/24 17:50:59 gene Exp $
 
 package Statistics::RankCorrelation;
-$VERSION = 0.0901;
+our $VERSION = '0.10';
 use strict;
 use warnings;
 use Carp;
 
 sub new {
     my $proto = shift;
-    my $class = ref ($proto) || $proto;
+    my $class = ref($proto) || $proto;
+
+    # Handle the unnamed single, pair and optional parameters.
+    my( $u, $v ) = ( shift, shift );
+    my %args = ();
+    # Two vectors and parameters given.
+    if( @_ ) {
+        %args = @_;
+    }
+    # Parameters given but no vectors.
+    elsif( $u =~ /^\D+$/ && $v ) {
+        $args{$u} = $v;
+    }
+    else {
+        # Empty object created.
+    }
+
     my $self  = {
-        x_data => shift || [],
-        y_data => shift || [],
+        x_data => $u || [],
+        y_data => $v || [],
+        sorted => $args{sorted},
     };
+
     bless $self, $class;
     $self->_init;
     return $self;
@@ -21,24 +39,35 @@ sub new {
 sub _init {
     my $self = shift;
 
-    # Automatically compute the statistical ranks if given vectors.
-    if( ( $self->{x_data} && $self->{y_data} ) &&
-        ( @{ $self->{x_data} } && @{ $self->{y_data} } )
+    # Automatically compute the statistical ranks if given data.
+    if( $self->x_data && $self->y_data &&
+        @{ $self->x_data } && @{ $self->y_data }
     ) {
-        # "co-normalize" the vectors.
-        ( $self->{x_data}, $self->{y_data} ) = pad_vectors(
-            $self->{x_data}, $self->{y_data}
-        );
+        # "Co-normalize" the vectors if they are unequal.
+        my( $x, $y ) = pad_vectors( $self->x_data, $self->y_data );
+
+        # "Co-sort" the vector pair by the first one.
+        ( $x, $y ) = co_sort( $x, $y ) if $self->{sorted};
+
+        # Set the massaged data.
+        $self->x_data( $x );
+        $self->y_data( $y );
+
+        # Set the size of the data vector.
+        $self->size( scalar @{ $self->x_data } );
 
         # Set the ranks of the vectors.
-        $self->x_rank( rank( $self->{x_data} ) );
-        $self->y_rank( rank( $self->{y_data} ) );
-
-        # Set the size of the unit data vector.
-        $self->{size} = @{ $self->{x_data} };
+        $self->x_rank( rank( $self->x_data ) );
+        $self->y_rank( rank( $self->y_data ) );
     }
 }
 
+sub size {
+    my $self = shift;
+    $self->{size} = shift if @_;
+    return $self->{size};
+}
+
 sub x_data {
     my $self = shift;
     $self->{x_data} = shift if @_;
@@ -63,26 +92,26 @@ sub y_rank {
     return $self->{y_rank};
 }
 
-# Return Spearman's rho correlation coefficient.
 sub spearman {
+# Return Spearman's rho correlation coefficient.
     my $self = shift;
 
     # Initialize the squared rank difference sum.
     my $sq_sum = 0;
 
-    # Compute the squared rank difference sum.
-    for( 0 .. $self->{size} - 1 ) {
-        $sq_sum += ( $self->{x_rank}[$_] - $self->{y_rank}[$_] ) ** 2;
+    # Compute the sum of the absolute difference of the squared ranks.
+    for( 0 .. $self->size - 1 ) {
+        $sq_sum += (abs(
+            $self->{x_rank}[$_] - $self->{y_rank}[$_]
+        )) ** 2;
 #warn "$sq_sum\n += ( $self->{x_rank}[$_] - $self->{y_rank}[$_] ) ** 2";
     }
 
-#warn "1 - ( (6 * $sq_sum) / ( $self->{size} * (( $self->{size} ** 2 ) - 1))\n";
-    return 1 - ( (6 * $sq_sum) /
-        ( $self->{size} * (( $self->{size} ** 2 ) - 1))
+#     warn "1 - ( (6 * $sq_sum) / ( $self->size * (( $self->size ** 2 ) - 1))\n";
+    return 1 - ( (6 * $sq_sum) / ( $self->size * (( $self->size ** 2 ) - 1))
     );
 }
 
-# Sort the given vectors as measurement pairs by the x data.
 
 sub rank {
     my $u = shift;
@@ -116,6 +145,21 @@ sub rank {
     return \@ranks;
 }
 
+sub co_sort {
+    my( $u, $v ) = @_;
+    return unless @$u == @$v;
+    # Ye olde Schwartzian Transforme:
+    $v = [
+        map { $_->[1] }
+            sort { $a->[0] <=> $b->[0] || $a->[1] <=> $b->[1] }
+                map { [ $u->[$_], $v->[$_] ] }
+                    0 .. @$u - 1
+    ];
+    # Sort the independent vector last.
+    $u = [ sort { $a <=> $b } @$u ];
+    return $u, $v;
+}
+
 sub csim {
     my $self = shift;
 
@@ -138,9 +182,9 @@ sub csim {
     return $k / (@$m1 * @$m1);
 }
 
+sub pad_vectors {
 # Append zeros to either vector for all values in the other that do
 # not have a corresponding value.
-sub pad_vectors {
     my ($u, $v) = @_;
 
     if (@$u > @$v) {
@@ -153,9 +197,9 @@ sub pad_vectors {
     return $u, $v;
 }
 
+sub correlation_matrix {
 # Build a square, binary matrix that represents "higher or lower"
 # value within the given vector.
-sub correlation_matrix {
     my $u = shift;
     my $c;
 
@@ -169,6 +213,36 @@ sub correlation_matrix {
     return $c;
 }
 
+sub kendall_tau {
+# Return Kendall's tau correlartion coefficient
+    my $self = shift;
+
+    # Initialize number of concordant and discordant values.
+    my $concordant = 0;
+    my $discordant = 0;
+
+    # Get a list of the order of the index of the sorted ranks of the
+    # first list.
+    my @list =
+    map $_->[0] + 1,
+        sort { $a->[1] <=> $b->[1] }
+            map [ $_, $self->{x_rank}[$_] ],
+                0 .. $self->size - 1;
+
+    # calculate number of concordant and number of discordant
+    while (my $index = shift @list) {
+        for (@list) {
+            $concordant++
+                if $self->{x_rank}[ $index - 1 ] < $self->{y_rank}[ $_ - 1 ];
+            $discordant++
+                if $self->{y_rank}[ $index - 1 ] > $self->{y_rank}[ $_ - 1 ] ;
+        }
+    }
+
+    return (2 * ($concordant - $discordant)) /
+           ($self->size * ($self->size - 1));
+}
+
 1;
 
 __END__
@@ -184,10 +258,17 @@ Statistics::RankCorrelation - Compute the rank correlation between two vectors
   $x = [ 8, 7, 6, 5, 4, 3, 2, 1 ];
   $y = [ 2, 1, 5, 3, 4, 7, 8, 6 ];
 
-  $c = Statistics::RankCorrelation->new( $x, $y );
+  $c = Statistics::RankCorrelation->new( $x, $y, sorted => 1 );
+
+  $s = $c->size;
+  $xd = $c->x_data;
+  $yd = $c->y_data;
+  $xr = $c->x_rank;
+  $yr = $c->y_rank;
 
   $n = $c->spearman;
   $m = $c->csim;
+  $t = $c->kendall_tau;
 
 =head1 DESCRIPTION
 
@@ -200,7 +281,7 @@ the module test file.
 Also the C<HANDY FUNCTIONS> section below has some ..handy functions 
 to use when computing sorted rank coefficients by hand.
 
-=head1 PUBLIC METHODS
+=head1 METHODS
 
 =head2 new
 
@@ -208,26 +289,47 @@ to use when computing sorted rank coefficients by hand.
 
 This method constructs a new C<Statistics::RankCorrelation> object.
 
-If given two numeric vectors (in the form of flat array references) as 
-arguments the object is initialized by computing the statistical ranks 
-of the vectors.  If they are of different cardinality the shorter 
-vector is first padded with trailing zeros.
+If given two numeric vectors (as array references), the object is
+initialized by computing the statistical ranks of the vectors.  If
+they are of different cardinality the shorter vector is first padded
+with trailing zeros.
 
-=head2 x_data, y_data
+=head2 x_data
 
+  $c->x_data( $y );
   $x = $c->x_data;
+
+Return and set the one dimensional array reference data.  This is the
+"unit" array, used as a reference for size and iteration.
+
+=head2 y_data
+
   $c->y_data( $y );
+  $x = $c->y_data;
 
-Return (and optionally set) the data samples that were provided to 
-the constructor as array references.
+Return and set the one dimensional array reference data.  This vector
+is dependent on the unit vector.
 
-=head2 x_rank, y_rank
+=head2 size
 
+  $c->size( $s );
+  $s = $c->size;
+
+Return and set the number of elements in the unit array.
+
+=head2 x_rank
+
+  $c->x_rank( $rx );
   $rx = $c->x_rank;
+
+Return and set the ranked "unit" data as an array reference.
+
+=head2 y_rank
+
+  $ry = $c->y_rank;
   $c->y_rank( $ry );
 
-Return (and optionally set) the statistically ranked data samples as 
-array references.
+Return (and optionally set) the ranked data as array references.
 
 =head2 spearman
 
@@ -239,12 +341,20 @@ case of the Pearson product-moment correlation.
 
 The formula is:
 
-      6 * sum( ( Xi - Yi ) ^ 2 )
+      6 * sum( (Xi - Yi)^2 )
   1 - --------------------------
-          N * ( N ^ 2 - 1 )
+           n (n^2 - 1)
 
 Where C<X> and C<Y> are the two rank vectors and C<i> is an index 
-from one to the C<N> number of samples.
+from one to C<n> number of samples.
+
+=head2 kendall_tau
+
+  $t = $c->kendall_tau;
+
+         4P
+  t = --------- - 1
+      n (n - 1)
 
 =head2 csim
 
@@ -259,7 +369,7 @@ values in the original vectors.
 
 This measure has been studied in musical contour analysis.
 
-=head1 HANDY FUNCTIONS
+=head1 FUNCTIONS
 
 =head2 rank
 
@@ -290,6 +400,13 @@ Append zeros to either input vector for all values in the other that
 do not have a corresponding value.  That is, "pad" the tail of the 
 shorter vector with zero values.
 
+=head2 co_sort
+
+  ( $u, $v ) = co_sort( $u, $v );
+
+Sort the vectors as two dimensional data-point pairs with B<u> values
+sorted first.
+
 =head2 correlation_matrix
 
   $matrix = correlation_matrix( $u );
@@ -301,34 +418,31 @@ or lower" value within the vector itself.
 
 =head1 TO DO
 
-Implement other rank correlation measures that are out there.
+Figure out what "> 0.5 discrepency" that Vladimir Babenko
+E<lt>Vl_Babenko@softhome.netE<gt> saw.
+
+Handle any number of vectors instead of just two.
+
+Implement other rank correlation measures that are out there...
 
 =head1 SEE ALSO
 
 For the C<csim> method:
 
-C<http://www2.mdanderson.org/app/ilya/Publications/JNMRcontour.pdf>
+L<http://www2.mdanderson.org/app/ilya/Publications/JNMRcontour.pdf>
 
-For the C<spearman> method:
+For the C<spearman> and C<kendall_tau> methods:
 
-C<http://mathworld.wolfram.com/SpearmanRankCorrelationCoefficient.html>
+L<http://mathworld.wolfram.com/SpearmanRankCorrelationCoefficient.html>
 
-C<http://faculty.vassar.edu/lowry/ch3b.html>
-
-C<http://www.pinkmonkey.com/studyguides/subjects/stats/chap6/s0606801.asp>
-
-C<http://fonsg3.let.uva.nl/Service/Statistics/RankCorrelation_coefficient.html>
-
-C<http://www.statsoftinc.com/textbook/stnonpar.html#correlations>
-
-C<http://www.analytics.washington.edu/~rossini/courses/intro-nonpar/text/Tied_Data.html>
-
-C<http://www.analytics.washington.edu/~rossini/courses/intro-nonpar/text/Spearman_s_tex2html_image_mark_tex2html_wrap_inline4049_.html>
+L<http://en.wikipedia.org/wiki/Kendall's_tau>
 
 =head1 THANK YOU
 
 Thomas Breslin E<lt>thomas@thep.lu.seE<gt> for unsorted C<rank> code.
 
+Jerome E<lt>jerome.hert@free.frE<gt> for Kendall's tau
+
 =head1 AUTHOR
 
 Gene Boggs E<lt>gene@cpan.orgE<gt>
